@@ -5,9 +5,9 @@ use warnings;
 use Log::ger;
 
 our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
-our $DATE = '2023-07-22'; # DATE
+our $DATE = '2024-01-25'; # DATE
 our $DIST = 'AppBase-Grep'; # DIST
-our $VERSION = '0.011'; # VERSION
+our $VERSION = '0.012'; # VERSION
 
 our %argspecs_files = (
     files => {
@@ -85,7 +85,8 @@ sub set_source_arg {
     $args->{_source} = sub {
       READ_LINE:
         {
-            if (!defined $fh) {
+            # open another file
+            if (!defined($fh) || $_[0]) {
                 return unless @files;
                 $file = shift @files;
                 log_trace "Opening $file ...";
@@ -122,7 +123,7 @@ AppBase::Grep::File - Resources for AppBase::Grep-based scripts that use file so
 
 =head1 VERSION
 
-This document describes version 0.011 of AppBase::Grep::File (from Perl distribution AppBase-Grep), released on 2023-07-22.
+This document describes version 0.012 of AppBase::Grep::File (from Perl distribution AppBase-Grep), released on 2024-01-25.
 
 =head1 FUNCTIONS
 
@@ -160,7 +161,7 @@ that are considered a bug and can be reported to me.
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2023, 2022, 2021, 2020, 2018 by perlancar <perlancar@cpan.org>.
+This software is copyright (c) 2024, 2023, 2022, 2021, 2020, 2018 by perlancar <perlancar@cpan.org>.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
